#!/usr/bin/python
# vim: set ai sts=4 sw=4 ts=8 expandtab:
'''Server Module
'''

from asyncore import dispatcher, loop
import time, threading, sys, socket

class Server:
    def __init__(self):
        self.objects = {}
        self.requests = []
        self.initObjects()

        self.poll()

    def poll(self):
        poller = self.poller = Poller(self)
        poller.listen()
        self.polling_thr = threading.Thread(target=poller)

    def initObjects(self):
        return 1

    def getObject(self, who):
        objects = self.objects
        if objects.has_key(who):
            return objects[who]
        else:
            return None

    def start(self):
        try:
            self.polling_thr.start()
            self.loop(0.5)
        except KeyboardInterrupt:
            self.dieSafely()

    def addRequest(self, second, ref):
        request_time = time.time() + second
        self.requests.append((request_time, ref))
        self.requests.sort()

    def loop(self, sec):
        requests = self.requests
        while 1:
            if len(requests) > 0:
                (request_time, ref) = requests[0]
                now = time.time()
                if now >= request_time:
                    requests.pop()
                    ref.signal('GO NULL')

            time.sleep(sec)

    def dieSafely(self):
        print('wait, server dying...')
        self.poller.stop()
        time.sleep(1)
        sys.exit(1)

class Poller:
    ''' 
    '''
    def __init__(self, parent, host='localhost', port=19999):
        connected = self.connected = {}
        self.disconnected = []

        self.parent = parent
        self.host = host
        self.port = port
        conn = self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        conn.bind((host, port))

    def __call__(self):
        self.loop()

    def listen(self):
        conn = self.conn
        connected = self.connected
        listener = self.listener = dispatcher(conn, connected)
        listener.set_reuse_addr()
        listener.listen(5)

    def getConnected(self):
        return self.connected
    def getDisconnected(self):
        return self.disconnected
    def getListener(self):
        return self.listener

    def loop(self):
        loop(1.0, map=self.connected)

    def stop(self):
        connected = self.connected
        for (fd, dispatcher) in connected.items():
            print('closing..')
            dispatcher.close()

def __test__():
    server = Server()
    server.start()

if __name__ == '__main__':
    __test__()

